#include "uniauthservice.h"
#include <QDebug>
#include <pwd.h>

#define UNIAUTH_DBUS_SERVICE      "org.ukui.UniauthBackend"
#define UNIAUTH_DBUS_PATH         "/org/ukui/UniauthBackend"
#define UNIAUTH_DBUS_INTERFACE    "org.ukui.UniauthBackend"

#define FD_DBUS_SERVICE     "org.freedesktop.DBus"
#define FD_DBUS_PATH        "/org/freedesktop/DBus"
#define FD_DBUS_INTERFACE   "org.freedesktop.DBus"

UniAuthService::UniAuthService(QObject *parent)
    : QDBusAbstractInterface(UNIAUTH_DBUS_SERVICE,
                             UNIAUTH_DBUS_PATH,
                             UNIAUTH_DBUS_INTERFACE,
                             QDBusConnection::systemBus(),
                             parent)
    , m_isActivatable(false)
{
    setTimeout(2147483647);
    QDBusInterface *dbusService = new QDBusInterface(FD_DBUS_SERVICE,
                                                   FD_DBUS_PATH,
                                                   FD_DBUS_INTERFACE,
                                                   QDBusConnection::systemBus());
    if (dbusService) {
        QDBusReply<QStringList> result = dbusService->call(QStringLiteral("ListActivatableNames"));
        if(!result.isValid()) {
            qWarning() << "ListActivatableNames error:" << result.error().message();
        } else {
            QStringList listNames = result.value();
            if (listNames.contains(UNIAUTH_DBUS_INTERFACE)) {
                m_isActivatable = true;
            }
        }
    }
}

// 设置默认设备
void UniAuthService::setDefaultDevice(int bioDevType, QString deviceName)
{
    qDebug() << " bioType : " << bioDevType << "deviceName : " << deviceName;
    QDBusMessage result = call(QStringLiteral("setDefaultDevice"), bioDevType, deviceName);
    if(result.type() == QDBusMessage::ErrorMessage)
    {
        qWarning() << "setDefaultDevice error:" << result.errorMessage();
        return ;
    }
    return ;
}

// 获取默认设备
QString UniAuthService::getDefaultDevice(QString userName, int bioDevType)
{
    QDBusMessage result = call(QStringLiteral("getDefaultDevice"), userName, bioDevType);
    if(result.type() == QDBusMessage::ErrorMessage)
    {
        qWarning() << "getDefaultDevice error:" << result.errorMessage();
        return "";
    }
    QList<QVariant> varResult = result.arguments();
    if (varResult.size() > 0) {
        QString strDefDevice = varResult.takeFirst().toString();
        return strDefDevice;
    } else {
        return "";
    }
}

//生物特征开关接口
bool UniAuthService::getBioAuthStatus(QString userName, int bioAuthType)
{
    QDBusMessage bioResult = call(QStringLiteral("getBioAuthStatus"), userName, bioAuthType);
    if(bioResult.type() == QDBusMessage::ErrorMessage)
    {
        qWarning() << "getBioStatus error:" << bioResult.errorMessage();
        return false;
    }
    QList<QVariant> varResult = bioResult.arguments();
    if (varResult.size() > 0) {
        return varResult.takeFirst().toBool();
    } else {
        return false;
    }
}

void UniAuthService::setBioAuthStatus(int bioAuthType, bool status)
{
    qDebug() << "setBioAuthStatus bioAuthType : " << bioAuthType << "status : " << status;
    QDBusMessage result = call(QStringLiteral("setBioAuthStatus"), bioAuthType, status);
    if(result.type() == QDBusMessage::ErrorMessage)
    {
        qWarning() << "setBioAuthStatus error:" << result.errorMessage();
        return ;
    }
    return ;
}

bool UniAuthService::isActivatable()
{
    return m_isActivatable;
}
